@@ -1,7 +1,7 @@
 /*
  *  connect.c
  *
- *  $Id: connect.c,v 1.25 2003/09/07 12:59:14 source Exp $
+ *  $Id: connect.c,v 1.31 2004/09/10 12:09:28 source Exp $
  *
  *  Connect (load) driver
  *
@@ -105,9 +105,9 @@ char *iodbc_version = VERSION;
 /*
  *  Prototypes
  */
-extern SQLRETURN _iodbcdm_driverunload ();
+extern SQLRETURN _iodbcdm_driverunload (HDBC hdbc);
 extern SQLRETURN SQL_API _iodbcdm_SetConnectOption (SQLHDBC hdbc,
-    SQLUSMALLINT fOption, SQLUINTEGER vParam, SQLCHAR waMode);
+    SQLUSMALLINT fOption, SQLULEN vParam, SQLCHAR waMode);
 
 extern char * _iodbcdm_getkeyvalinstr (char *cnstr, int cnlen,
     char *keywd, char *value, int size);
@@ -116,13 +116,14 @@ extern wchar_t * _iodbcdm_getkeyvalinstrw (wchar_t *cnstr, int cnlen,
 
 
 static SQLRETURN
-_iodbcdm_SetConnectOption_init (SQLHDBC hdbc,
-    SQLUSMALLINT fOption,
-    SQLUINTEGER vParam,
-    UCHAR waMode)
+_iodbcdm_SetConnectOption_init (
+    SQLHDBC		  hdbc,
+    SQLUSMALLINT	  fOption,
+    SQLULEN		  vParam,
+    UCHAR		  waMode)
 {
   CONN (pdbc, hdbc);
-  ENV_t FAR *penv = pdbc->henv;
+  ENVR (penv, pdbc->henv);
   HPROC hproc = SQL_NULL_HPROC;
   SQLRETURN retcode = SQL_SUCCESS;
 
@@ -229,14 +230,16 @@ _iodbcdm_getInfo_init (SQLHDBC hdbc,
     SQLUSMALLINT fInfoType,
     SQLPOINTER rgbInfoValue,
     SQLSMALLINT cbInfoValueMax,
-    SQLSMALLINT FAR * pcbInfoValue,
+    SQLSMALLINT * pcbInfoValue,
     SQLCHAR waMode)
 {
   CONN (pdbc, hdbc);
-  ENV_t FAR *penv = pdbc->henv;
+  ENVR (penv, pdbc->henv);
   HPROC hproc = SQL_NULL_HPROC;
   SQLRETURN retcode = SQL_SUCCESS;
 
+  waMode = waMode; /*UNUSED*/
+
   switch(fInfoType)
     {
       case SQL_CURSOR_COMMIT_BEHAVIOR:
@@ -273,18 +276,19 @@ _iodbcdm_getInfo_init (SQLHDBC hdbc,
  */
 SQLRETURN
 _iodbcdm_driverload (
-    char FAR * path,
+    char * path,
     HDBC hdbc,
     SWORD thread_safe,
+    SWORD unload_safe,
     UCHAR waMode)
 {
   CONN (pdbc, hdbc);
-  GENV_t FAR *genv;
-  ENV_t FAR *penv = NULL;
+  ENVR (penv, NULL);
+  GENV (genv, NULL);
   HDLL hdll;
   HPROC hproc;
   SQLRETURN retcode = SQL_SUCCESS;
-  int sqlstat = en_00000;
+  sqlstcode_t sqlstat = en_00000;
 
   if (path == NULL || ((char*)path)[0] == '\0')
     {
@@ -297,10 +301,14 @@ _iodbcdm_driverload (
       return SQL_INVALID_HANDLE;
     }
 
-  genv = (GENV_t FAR *) pdbc->genv;
+  genv = (GENV_t *) pdbc->genv;
 
   /* This will either load the driver dll or increase its reference count */
-  hdll = _iodbcdm_dllopen ((char FAR *) path);
+  hdll = _iodbcdm_dllopen ((char *) path);
+  
+  /* Set flag if it is safe to unload the driver after use */
+  if (unload_safe)
+    _iodbcdm_safe_unload (hdll);
 
   if (hdll == SQL_NULL_HDLL)
     {
@@ -309,7 +317,7 @@ _iodbcdm_driverload (
       return SQL_ERROR;
     }
 
-  penv = (ENV_t FAR *) (pdbc->henv);
+  penv = (ENV_t *) (pdbc->henv);
 
   if (penv != NULL)
     {
@@ -334,9 +342,9 @@ _iodbcdm_driverload (
        * find out whether this dll has already been loaded on another
        * connection
        */
-      for (penv = (ENV_t FAR *) genv->henv;
+      for (penv = (ENV_t *) genv->henv;
 	  penv != NULL;
-	  penv = (ENV_t FAR *) penv->next)
+	  penv = (ENV_t *) penv->next)
 	{
 	  if (penv->hdll == hdll)
 	    {
@@ -355,7 +363,7 @@ _iodbcdm_driverload (
 	  int i;
 
 	  /* create a new dll env instance */
-	  penv = (ENV_t FAR *) MEM_ALLOC (sizeof (ENV_t));
+	  penv = (ENV_t *) MEM_ALLOC (sizeof (ENV_t));
 
 	  if (penv == NULL)
 	    {
@@ -461,11 +469,11 @@ _iodbcdm_driverload (
 	    }
 
 	  /* insert into dll env list */
-	  penv->next = (ENV_t FAR *) genv->henv;
+	  penv->next = (ENV_t *) genv->henv;
 	  genv->henv = penv;
 
 	  /* initiate this new env entry */
-	  penv->refcount = 1;	/* we will increase it after
+	  penv->refcount = 0;	/* we will increase it after
 				 * driver's SQLAllocConnect()
 				 * success
 				 */
@@ -570,9 +578,9 @@ SQLRETURN
 _iodbcdm_driverunload (HDBC hdbc)
 {
   CONN (pdbc, hdbc);
-  ENV_t FAR *penv;
-  ENV_t FAR *tpenv;
-  GENV_t FAR *genv;
+  ENVR (penv, pdbc->henv);
+  GENV (genv, pdbc->genv);
+  ENV_t *tpenv;
   HPROC hproc;
   SQLRETURN retcode = SQL_SUCCESS;
 
@@ -581,10 +589,6 @@ _iodbcdm_driverunload (HDBC hdbc)
       return SQL_INVALID_HANDLE;
     }
 
-  /* no pointer check will be performed in this function */
-  penv = (ENV_t FAR *) pdbc->henv;
-  genv = (GENV_t FAR *) pdbc->genv;
-
   if (penv == NULL || penv->hdll == SQL_NULL_HDLL)
     {
       return SQL_SUCCESS;
@@ -646,9 +650,9 @@ _iodbcdm_driverunload (HDBC hdbc)
 
       penv->hdll = SQL_NULL_HDLL;
 
-      for (tpenv = (ENV_t FAR *) genv->henv;
+      for (tpenv = (ENV_t *) genv->henv;
 	  tpenv != NULL;
-	  tpenv = (ENV_t FAR *) penv->next)
+	  tpenv = (ENV_t *) penv->next)
 	{
 	  if (tpenv == penv)
 	    {
@@ -720,7 +724,7 @@ _iodbcdm_dbcdelayset (HDBC hdbc, UCHAR waMode)
   if (pdbc->current_qualifier != NULL)
     {
       ret = _iodbcdm_SetConnectOption_init (hdbc, SQL_CURRENT_QUALIFIER,
-	      (SQLUINTEGER)pdbc->current_qualifier,
+	      (SQLULEN) pdbc->current_qualifier,
 	      pdbc->current_qualifier_WA);
 
       retcode |= ret;
@@ -781,14 +785,19 @@ _iodbcdm_dbcdelayset (HDBC hdbc, UCHAR waMode)
 
 
 static SQLRETURN
-_iodbcdm_con_settracing (HDBC hdbc, char *dsn, int dsnlen, UCHAR waMode)
+_iodbcdm_con_settracing (HDBC hdbc, SQLCHAR *dsn, int dsnlen, UCHAR waMode)
 {
   SQLUINTEGER trace = SQL_OPT_TRACE_OFF;
   char buf[1024];
 
+  /* Unused */
+  hdbc=hdbc;
+  dsnlen=dsnlen;
+  waMode = waMode;
+
   /* Get the TraceFile keyword value from the ODBC section */
   SQLSetConfigMode (ODBC_BOTH_DSN);
-  if ((SQLGetPrivateProfileString (dsn, "TraceFile", "", 
+  if ((SQLGetPrivateProfileString ((char *) dsn, "TraceFile", "", 
 	buf, sizeof (buf), "odbc.ini") == 0 || !buf[0]))
     STRCPY (buf, SQL_OPT_TRACE_FILE_DEFAULT);
 
@@ -796,7 +805,7 @@ _iodbcdm_con_settracing (HDBC hdbc, char *dsn, int dsnlen, UCHAR waMode)
 
   /* Get the Trace keyword value from the ODBC section */
   SQLSetConfigMode (ODBC_BOTH_DSN);
-  if (SQLGetPrivateProfileString (dsn, "Trace", "", 
+  if (SQLGetPrivateProfileString ((char *) dsn, "Trace", "", 
 	buf, sizeof (buf), "odbc.ini")
       && (STRCASEEQ (buf, "on") || STRCASEEQ (buf, "yes")
 	  || STRCASEEQ (buf, "1")))
@@ -824,18 +833,19 @@ SQLConnect_Internal (SQLHDBC hdbc,
     SQLCHAR waMode)
 {
   CONN (pdbc, hdbc);
+  ENVR (penv, NULL);
   SQLRETURN retcode = SQL_SUCCESS;
   SQLRETURN setopterr = SQL_SUCCESS;
   /* MS SDK Guide specifies driver path can't longer than 255. */
   char driver[1024] = { '\0' };
   char buf[256];
-  ENV_t FAR *penv = NULL;
   HPROC hproc = SQL_NULL_HPROC;
   SWORD thread_safe;
+  SWORD unload_safe;
   void * _szDSN = NULL;
   void * _szUID = NULL;
   void * _szAuthStr = NULL;
-  char *_dsn = szDSN;
+  SQLCHAR *_dsn = (SQLCHAR *) szDSN;
   SQLSMALLINT _dsn_len = cbDSN;
 
   /* check arguments */
@@ -864,7 +874,8 @@ SQLConnect_Internal (SQLHDBC hdbc,
 
   if (waMode == 'W')
     {
-      _dsn = _szDSN = dm_SQL_WtoU8((SQLWCHAR *)szDSN, cbDSN);
+      _szDSN = (void *) dm_SQL_WtoU8((SQLWCHAR *)szDSN, cbDSN);
+      _dsn = (SQLCHAR *) _szDSN;
       _dsn_len = SQL_NTS;
       if (_dsn == NULL)
         {
@@ -881,24 +892,38 @@ SQLConnect_Internal (SQLHDBC hdbc,
     }
 
   /*
-   *  Get the name of the driver module and load it
+   *  Check whether driver is thread safe
    */
   thread_safe = 1;		/* Assume driver is thread safe */
 
   SQLSetConfigMode (ODBC_BOTH_DSN);
-  if ( SQLGetPrivateProfileString (_dsn, "ThreadManager", "", 
+  if ( SQLGetPrivateProfileString ((char *) _dsn, "ThreadManager", "", 
 	buf, sizeof(buf), "odbc.ini") &&
       (STRCASEEQ (buf, "on") || STRCASEEQ (buf, "1")))
     {
       thread_safe = 0;	/* Driver needs a thread manager */
     }
 
+  /*
+   *  Check if it is safe to unload the driver
+   */
+  unload_safe = 0;		/* Assume driver is not unload safe */
+
+  SQLSetConfigMode (ODBC_BOTH_DSN);
+  if ( SQLGetPrivateProfileString ((char *) _dsn, "UnloadSafe", "", 
+	buf, sizeof(buf), "odbc.ini") &&
+      (STRCASEEQ (buf, "on") || STRCASEEQ (buf, "1")))
+    {
+      unload_safe = 1;
+    }
+
+
   /*
    *  Get the name of the driver module and load it
    */
   SQLSetConfigMode (ODBC_BOTH_DSN);
-  if ( SQLGetPrivateProfileString (_dsn, "Driver", "", 
-	driver, sizeof(driver), "odbc.ini") == 0)
+  if ( SQLGetPrivateProfileString ((char *) _dsn, "Driver", "", 
+	(char *) driver, sizeof(driver), "odbc.ini") == 0)
     /* No specified or default dsn section or
      * no driver specification in this dsn section */
     {
@@ -911,7 +936,7 @@ SQLConnect_Internal (SQLHDBC hdbc,
   MEM_FREE(_szDSN);
   _szDSN = NULL;
 
-  retcode = _iodbcdm_driverload ((char FAR *)driver, pdbc, thread_safe, waMode);
+  retcode = _iodbcdm_driverload ((char *)driver, pdbc, thread_safe, unload_safe, waMode);
 
   switch (retcode)
     {
@@ -932,7 +957,7 @@ SQLConnect_Internal (SQLHDBC hdbc,
       return retcode;
     }
 
-  penv = pdbc->henv;
+  penv = (ENV_t *) pdbc->henv;
 
   if ((penv->unicode_driver && waMode != 'W')
       || (!penv->unicode_driver && waMode == 'W'))
@@ -1009,11 +1034,11 @@ SQLConnect_Internal (SQLHDBC hdbc,
 SQLRETURN SQL_API
 SQLConnect (
   SQLHDBC		  hdbc,
-  SQLCHAR FAR		* szDSN,
+  SQLCHAR 		* szDSN,
   SQLSMALLINT		  cbDSN,
-  SQLCHAR FAR		* szUID,
+  SQLCHAR 		* szUID,
   SQLSMALLINT		  cbUID,
-  SQLCHAR FAR		* szAuthStr,
+  SQLCHAR 		* szAuthStr,
   SQLSMALLINT		  cbAuthStr)
 {
   ENTER_HDBC (hdbc, 1,
@@ -1041,11 +1066,11 @@ SQLConnect (
 SQLRETURN SQL_API
 SQLConnectA (
   SQLHDBC		  hdbc,
-  SQLCHAR FAR		* szDSN,
+  SQLCHAR 		* szDSN,
   SQLSMALLINT		  cbDSN,
-  SQLCHAR FAR		* szUID,
+  SQLCHAR 		* szUID,
   SQLSMALLINT		  cbUID,
-  SQLCHAR FAR		* szAuthStr,
+  SQLCHAR 		* szAuthStr,
   SQLSMALLINT		  cbAuthStr)
 {
   ENTER_HDBC (hdbc, 1,
@@ -1072,11 +1097,11 @@ SQLConnectA (
 
 SQLRETURN SQL_API
 SQLConnectW (SQLHDBC hdbc,
-    SQLWCHAR FAR * szDSN,
+    SQLWCHAR * szDSN,
     SQLSMALLINT cbDSN,
-    SQLWCHAR FAR * szUID,
+    SQLWCHAR * szUID,
     SQLSMALLINT cbUID,
-    SQLWCHAR FAR * szAuthStr,
+    SQLWCHAR * szAuthStr,
     SQLSMALLINT cbAuthStr)
 {
   ENTER_HDBC (hdbc, 1,
@@ -1115,6 +1140,7 @@ SQLDriverConnect_Internal (
     SQLCHAR waMode)
 {
   CONN (pdbc, hdbc);
+  ENVR (penv, NULL);
   HDLL hdll;
   void *drv;
   SQLWCHAR drvbuf[1024];
@@ -1122,8 +1148,8 @@ SQLDriverConnect_Internal (
   SQLWCHAR dsnbuf[SQL_MAX_DSN_LENGTH + 1];
   SQLWCHAR prov[1024];
   SWORD thread_safe;
+  SWORD unload_safe;
   char buf[1024];
-  ENV_t FAR *penv = NULL;
   HPROC hproc = SQL_NULL_HPROC;
   void *_ConnStrIn = NULL;
   void *_ConnStrOut = NULL;
@@ -1135,7 +1161,7 @@ SQLDriverConnect_Internal (
 
   HPROC dialproc = SQL_NULL_HPROC;
 
-  int sqlstat = en_00000;
+  sqlstcode_t sqlstat = en_00000;
   SQLRETURN retcode = SQL_SUCCESS;
   SQLRETURN setopterr = SQL_SUCCESS;
 
@@ -1159,16 +1185,16 @@ SQLDriverConnect_Internal (
 
   if (waMode != 'W')
     {
-      drv = _iodbcdm_getkeyvalinstr (szConnStrIn, cbConnStrIn,
+      drv = _iodbcdm_getkeyvalinstr ((char *)szConnStrIn, cbConnStrIn,
 	  "DRIVER", (char *) drvbuf, sizeof (drvbuf));
-      dsn = _iodbcdm_getkeyvalinstr (szConnStrIn, cbConnStrIn,
+      dsn = _iodbcdm_getkeyvalinstr ((char *)szConnStrIn, cbConnStrIn,
 	  "DSN", (char *) dsnbuf, sizeof (dsnbuf));
     }
   else
     {
-      drv = _iodbcdm_getkeyvalinstrw (szConnStrIn, cbConnStrIn,
+      drv = _iodbcdm_getkeyvalinstrw ((wchar_t *)szConnStrIn, cbConnStrIn,
 	  L"DRIVER", drvbuf, sizeof (drvbuf) / sizeof (SQLWCHAR));
-      dsn = _iodbcdm_getkeyvalinstrw (szConnStrIn, cbConnStrIn,
+      dsn = _iodbcdm_getkeyvalinstrw ((wchar_t *)szConnStrIn, cbConnStrIn,
 	  L"DSN", dsnbuf, sizeof (dsnbuf) / sizeof (SQLWCHAR));
     }
 
@@ -1281,12 +1307,12 @@ SQLDriverConnect_Internal (
 
   if (dsn == NULL || *(char *) dsn == '\0')
     {
-      dsn = "default";
+      dsn = (void *) "default";
     }
   else
     /* if you want tracing, you must use a DSN */
     {
-      setopterr |= _iodbcdm_con_settracing (pdbc, dsn, SQL_NTS, waMode);
+      setopterr |= _iodbcdm_con_settracing (pdbc, (SQLCHAR *) dsn, SQL_NTS, waMode);
     }
 
   /*
@@ -1295,20 +1321,33 @@ SQLDriverConnect_Internal (
   thread_safe = 1;		/* Assume driver is thread safe */
 
   SQLSetConfigMode (ODBC_BOTH_DSN);
-  if (SQLGetPrivateProfileString (dsn, "ThreadManager", "", 
+  if (SQLGetPrivateProfileString ((char *) dsn, "ThreadManager", "", 
 	buf, sizeof (buf), "odbc.ini")
       && (STRCASEEQ (buf, "on") || STRCASEEQ (buf, "1")))
     {
       thread_safe = 0;		/* Driver needs a thread manager */
     }
 
+  /*
+   *  Check whether driver is unload safe
+   */
+  unload_safe = 0;		/* Assume driver is not unload safe */
+
+  SQLSetConfigMode (ODBC_BOTH_DSN);
+  if (SQLGetPrivateProfileString ((char *) dsn, "UnloadSafe", "", 
+	buf, sizeof (buf), "odbc.ini")
+      && (STRCASEEQ (buf, "on") || STRCASEEQ (buf, "1")))
+    {
+      unload_safe = 1;
+    }
+
   /*
    *  Get the name of the driver module
    */
   if (drv == NULL || *(char *) drv == '\0')
     {
       SQLSetConfigMode (ODBC_BOTH_DSN);
-      if (SQLGetPrivateProfileString (dsn, "Driver", "", 
+      if (SQLGetPrivateProfileString ((char *) dsn, "Driver", "", 
 		buf, sizeof (buf), "odbc.ini") != 0)
 	{
 	  drv = buf;
@@ -1323,7 +1362,7 @@ SQLDriverConnect_Internal (
       return SQL_ERROR;
     }
 
-  retcode = _iodbcdm_driverload (drv, pdbc, thread_safe, waMode);
+  retcode = _iodbcdm_driverload ((char *) drv, pdbc, thread_safe, unload_safe, waMode);
 
   MEM_FREE (_dsn_u8);
   MEM_FREE (_drv_u8);
@@ -1347,7 +1386,7 @@ SQLDriverConnect_Internal (
       return retcode;
     }
 
-  penv = pdbc->henv;
+  penv = (ENV_t *) pdbc->henv;
 
   if ((penv->unicode_driver && waMode != 'W')
       || (!penv->unicode_driver && waMode == 'W'))
@@ -1406,12 +1445,12 @@ SQLDriverConnect_Internal (
       if (waMode != 'W')
 	{
 	  /* ansi<=unicode */
-          dm_StrCopyOut2_W2A (connStrOut, szConnStrOut, cbConnStrOutMax, NULL);
+          dm_StrCopyOut2_W2A ((SQLWCHAR *) connStrOut, (SQLCHAR *) szConnStrOut, cbConnStrOutMax, NULL);
 	}
       else
 	{
 	  /* unicode<=ansi */
-          dm_StrCopyOut2_A2W (connStrOut, szConnStrOut, cbConnStrOutMax, NULL);
+          dm_StrCopyOut2_A2W ((SQLCHAR *) connStrOut, (SQLWCHAR *) szConnStrOut, cbConnStrOutMax, NULL);
 	}
     }
 
@@ -1447,11 +1486,11 @@ end:
 SQLRETURN SQL_API
 SQLDriverConnect (SQLHDBC hdbc,
     SQLHWND hwnd,
-    SQLCHAR FAR * szConnStrIn,
+    SQLCHAR * szConnStrIn,
     SQLSMALLINT cbConnStrIn,
-    SQLCHAR FAR * szConnStrOut,
+    SQLCHAR * szConnStrOut,
     SQLSMALLINT cbConnStrOutMax,
-    SQLSMALLINT FAR * pcbConnStrOut,
+    SQLSMALLINT * pcbConnStrOut,
     SQLUSMALLINT fDriverCompletion)
 {
   ENTER_HDBC (hdbc, 1,
@@ -1483,11 +1522,11 @@ SQLDriverConnect (SQLHDBC hdbc,
 SQLRETURN SQL_API
 SQLDriverConnectA (SQLHDBC hdbc,
     SQLHWND hwnd,
-    SQLCHAR FAR * szConnStrIn,
+    SQLCHAR * szConnStrIn,
     SQLSMALLINT cbConnStrIn,
-    SQLCHAR FAR * szConnStrOut,
+    SQLCHAR * szConnStrOut,
     SQLSMALLINT cbConnStrOutMax,
-    SQLSMALLINT FAR * pcbConnStrOut,
+    SQLSMALLINT * pcbConnStrOut,
     SQLUSMALLINT fDriverCompletion)
 {
   ENTER_HDBC (hdbc, 1,
@@ -1519,11 +1558,11 @@ SQLDriverConnectA (SQLHDBC hdbc,
 SQLRETURN SQL_API
 SQLDriverConnectW (SQLHDBC hdbc,
     SQLHWND hwnd,
-    SQLWCHAR FAR * szConnStrIn,
+    SQLWCHAR * szConnStrIn,
     SQLSMALLINT cbConnStrIn,
-    SQLWCHAR FAR * szConnStrOut,
+    SQLWCHAR * szConnStrOut,
     SQLSMALLINT cbConnStrOutMax,
-    SQLSMALLINT FAR * pcbConnStrOut,
+    SQLSMALLINT * pcbConnStrOut,
     SQLUSMALLINT fDriverCompletion)
 {
   ENTER_HDBC (hdbc, 1,
@@ -1557,16 +1596,17 @@ SQLBrowseConnect_Internal (SQLHDBC hdbc,
     SQLPOINTER szConnStrIn,
     SQLSMALLINT cbConnStrIn,
     SQLPOINTER szConnStrOut,
-    SQLSMALLINT cbConnStrOutMax, SQLSMALLINT FAR * pcbConnStrOut,
+    SQLSMALLINT cbConnStrOutMax, SQLSMALLINT * pcbConnStrOut,
     SQLCHAR waMode)
 {
   CONN (pdbc, hdbc);
+  ENVR (penv, NULL);
   void *drv, *dsn;
   char drvbuf[4096];
   char dsnbuf[SQL_MAX_DSN_LENGTH * UTF8_MAX_CHAR_LEN + 1];
   char buf[1024];
   SWORD thread_safe;
-  ENV_t FAR *penv = NULL;
+  SWORD unload_safe;
   HPROC hproc = SQL_NULL_HPROC;
   void * _ConnStrIn = NULL;
   void * _ConnStrOut = NULL;
@@ -1585,19 +1625,19 @@ SQLBrowseConnect_Internal (SQLHDBC hdbc,
 
   if (pdbc->state == en_dbc_allocated)
     {
-        drv = _iodbcdm_getkeyvalinstr (szConnStrIn, cbConnStrIn,
+        drv = _iodbcdm_getkeyvalinstr ((char *) szConnStrIn, cbConnStrIn,
 	  "DRIVER", (char*)drvbuf, sizeof (drvbuf));
 
-        dsn = _iodbcdm_getkeyvalinstr (szConnStrIn, cbConnStrIn,
+        dsn = _iodbcdm_getkeyvalinstr ((char *) szConnStrIn, cbConnStrIn,
 	  "DSN", (char*)dsnbuf, sizeof (dsnbuf));
 
 
         if (dsn == NULL || ((char*)dsn)[0] == '\0')
-          dsn = "default";
+          dsn = (void *) "default";
         else
           /* if you want tracing, you must use a DSN */
           {
-	    if (_iodbcdm_con_settracing (pdbc, dsn, SQL_NTS, waMode) == SQL_ERROR)
+	    if (_iodbcdm_con_settracing (pdbc, (SQLCHAR *) dsn, SQL_NTS, waMode) == SQL_ERROR)
 	      {
 	        return SQL_ERROR;
 	      }
@@ -1609,20 +1649,33 @@ SQLBrowseConnect_Internal (SQLHDBC hdbc,
         thread_safe = 1;		/* Assume driver is thread safe */
 
         SQLSetConfigMode (ODBC_BOTH_DSN);
-        if ( SQLGetPrivateProfileString (dsn, "ThreadManager", "", 
+        if ( SQLGetPrivateProfileString ((char *) dsn, "ThreadManager", "", 
 		buf, sizeof(buf), "odbc.ini") &&
             (STRCASEEQ (buf, "on") || STRCASEEQ (buf, "1")))
           {
             thread_safe = 0;	/* Driver needs a thread manager */
           }
 
+        /*
+         *  Check whether driver is unload safe
+         */
+        unload_safe = 0;		/* Assume driver is not unload safe */
+
+        SQLSetConfigMode (ODBC_BOTH_DSN);
+        if ( SQLGetPrivateProfileString ((char *) dsn, "ThreadManager", "", 
+		buf, sizeof(buf), "odbc.ini") &&
+            (STRCASEEQ (buf, "on") || STRCASEEQ (buf, "1")))
+          {
+            unload_safe = 1;
+          }
+
         /*
          *  Get the name of the driver module and load it
          */
         if (drv == NULL || *(char*)drv == '\0')
           {
             SQLSetConfigMode (ODBC_BOTH_DSN);
-            if ( SQLGetPrivateProfileString (dsn, "Driver", "", 
+            if ( SQLGetPrivateProfileString ((char *) dsn, "Driver", "", 
 		buf, sizeof(buf), "odbc.ini") != 0)
               {
                 drv = buf;
@@ -1635,7 +1688,7 @@ SQLBrowseConnect_Internal (SQLHDBC hdbc,
 	  return SQL_ERROR;
 	}
 
-      retcode = _iodbcdm_driverload (drv, pdbc, thread_safe, waMode);
+      retcode = _iodbcdm_driverload ((char *) drv, pdbc, thread_safe, unload_safe, waMode);
 
       switch (retcode)
 	{
@@ -1662,7 +1715,7 @@ SQLBrowseConnect_Internal (SQLHDBC hdbc,
       return SQL_ERROR;
     }
 
-  penv = pdbc->henv;
+  penv = (ENV_t *) pdbc->henv;
 
   if ((penv->unicode_driver && waMode != 'W')
       || (!penv->unicode_driver && waMode == 'W'))
@@ -1720,12 +1773,12 @@ SQLBrowseConnect_Internal (SQLHDBC hdbc,
       if (waMode != 'W')
         {
         /* ansi<=unicode*/
-          dm_StrCopyOut2_W2A (connStrOut, szConnStrOut, cbConnStrOutMax, NULL);
+          dm_StrCopyOut2_W2A ((SQLWCHAR *) connStrOut, (SQLCHAR *) szConnStrOut, cbConnStrOutMax, NULL);
         }
       else
         {
         /* unicode<=ansi*/
-          dm_StrCopyOut2_A2W (connStrOut, szConnStrOut, cbConnStrOutMax, NULL);
+          dm_StrCopyOut2_A2W ((SQLCHAR *) connStrOut, (SQLWCHAR *) szConnStrOut, cbConnStrOutMax, NULL);
         }
     }
 
@@ -1765,11 +1818,11 @@ SQLBrowseConnect_Internal (SQLHDBC hdbc,
 
 SQLRETURN SQL_API
 SQLBrowseConnect (SQLHDBC hdbc,
-    SQLCHAR FAR * szConnStrIn,
+    SQLCHAR * szConnStrIn,
     SQLSMALLINT cbConnStrIn,
-    SQLCHAR FAR * szConnStrOut,
+    SQLCHAR * szConnStrOut,
     SQLSMALLINT cbConnStrOutMax,
-    SQLSMALLINT FAR * pcbConnStrOut)
+    SQLSMALLINT * pcbConnStrOut)
 {
   ENTER_HDBC (hdbc, 1,
     trace_SQLBrowseConnect (TRACE_ENTER,
@@ -1793,11 +1846,11 @@ SQLBrowseConnect (SQLHDBC hdbc,
 
 SQLRETURN SQL_API
 SQLBrowseConnectA (SQLHDBC hdbc,
-    SQLCHAR FAR * szConnStrIn,
+    SQLCHAR * szConnStrIn,
     SQLSMALLINT cbConnStrIn,
-    SQLCHAR FAR * szConnStrOut,
+    SQLCHAR * szConnStrOut,
     SQLSMALLINT cbConnStrOutMax,
-    SQLSMALLINT FAR * pcbConnStrOut)
+    SQLSMALLINT * pcbConnStrOut)
 {
   ENTER_HDBC (hdbc, 1,
     trace_SQLBrowseConnect (TRACE_ENTER,
@@ -1821,11 +1874,11 @@ SQLBrowseConnectA (SQLHDBC hdbc,
 
 SQLRETURN SQL_API
 SQLBrowseConnectW (SQLHDBC hdbc,
-    SQLWCHAR FAR * szConnStrIn,
+    SQLWCHAR * szConnStrIn,
     SQLSMALLINT cbConnStrIn,
-    SQLWCHAR FAR * szConnStrOut,
+    SQLWCHAR * szConnStrOut,
     SQLSMALLINT cbConnStrOutMax,
-    SQLSMALLINT FAR * pcbConnStrOut)
+    SQLSMALLINT * pcbConnStrOut)
 {
   ENTER_HDBC (hdbc, 1,
     trace_SQLBrowseConnectW (TRACE_ENTER,
@@ -1851,11 +1904,11 @@ static SQLRETURN
 SQLDisconnect_Internal (SQLHDBC hdbc)
 {
   CONN (pdbc, hdbc);
-  STMT_t FAR *pstmt;
+  STMT (pstmt, NULL);
   SQLRETURN retcode;
   HPROC hproc = SQL_NULL_HPROC;
 
-  int sqlstat = en_00000;
+  sqlstcode_t sqlstat = en_00000;
 
   /* check hdbc state */
   if (pdbc->state == en_dbc_allocated)
@@ -1864,9 +1917,9 @@ SQLDisconnect_Internal (SQLHDBC hdbc)
     }
 
   /* check stmt(s) state */
-  for (pstmt = (STMT_t FAR *) pdbc->hstmt;
+  for (pstmt = (STMT_t *) pdbc->hstmt;
       pstmt != NULL && sqlstat == en_00000;
-      pstmt = (STMT_t FAR *) pstmt->next)
+      pstmt = (STMT_t *) pstmt->next)
     {
       if (pstmt->state >= en_stmt_needdata
 	  || pstmt->asyn_on != en_NullProc)
@@ -1945,13 +1998,13 @@ SQLNativeSql_Internal (SQLHDBC hdbc,
     SQLINTEGER cbSqlStrIn,
     SQLPOINTER szSqlStr,
     SQLINTEGER cbSqlStrMax,
-    SQLINTEGER FAR * pcbSqlStr,
+    SQLINTEGER * pcbSqlStr,
     SQLCHAR waMode)
 {
   CONN (pdbc, hdbc);
-  int sqlstat = en_00000;
+  ENVR (penv, pdbc->henv);
+  sqlstcode_t sqlstat = en_00000;
   SQLRETURN retcode = SQL_SUCCESS;
-  ENV_t FAR *penv = pdbc->henv;
   HPROC hproc = SQL_NULL_HPROC;
   void * _SqlStrIn = NULL;
   void * _SqlStr = NULL;
@@ -2038,12 +2091,12 @@ SQLNativeSql_Internal (SQLHDBC hdbc,
       if (waMode != 'W')
         {
         /* ansi<=unicode*/
-          dm_StrCopyOut2_W2A (sqlStr, szSqlStr, cbSqlStrMax, NULL);
+          dm_StrCopyOut2_W2A ((SQLWCHAR *) sqlStr, (SQLCHAR *) szSqlStr, cbSqlStrMax, NULL);
         }
       else
         {
         /* unicode<=ansi*/
-          dm_StrCopyOut2_A2W (sqlStr, szSqlStr, cbSqlStrMax, NULL);
+          dm_StrCopyOut2_A2W ((SQLCHAR *) sqlStr, (SQLWCHAR *) szSqlStr, cbSqlStrMax, NULL);
         }
     }
 
@@ -2056,11 +2109,11 @@ SQLNativeSql_Internal (SQLHDBC hdbc,
 SQLRETURN SQL_API
 SQLNativeSql (
     SQLHDBC hdbc,
-    SQLCHAR FAR * szSqlStrIn,
+    SQLCHAR * szSqlStrIn,
     SQLINTEGER cbSqlStrIn,
-    SQLCHAR FAR * szSqlStr,
+    SQLCHAR * szSqlStr,
     SQLINTEGER cbSqlStrMax,
-    SQLINTEGER FAR * pcbSqlStr)
+    SQLINTEGER * pcbSqlStr)
 {
   ENTER_HDBC (hdbc, 0,
     trace_SQLNativeSql (TRACE_ENTER,
@@ -2085,11 +2138,11 @@ SQLNativeSql (
 SQLRETURN SQL_API
 SQLNativeSqlA (
     SQLHDBC hdbc,
-    SQLCHAR FAR * szSqlStrIn,
+    SQLCHAR * szSqlStrIn,
     SQLINTEGER cbSqlStrIn,
-    SQLCHAR FAR * szSqlStr,
+    SQLCHAR * szSqlStr,
     SQLINTEGER cbSqlStrMax,
-    SQLINTEGER FAR * pcbSqlStr)
+    SQLINTEGER * pcbSqlStr)
 {
   ENTER_HDBC (hdbc, 0,
     trace_SQLNativeSql (TRACE_ENTER,
@@ -2114,11 +2167,11 @@ SQLNativeSqlA (
 SQLRETURN SQL_API
 SQLNativeSqlW (
     SQLHDBC hdbc,
-    SQLWCHAR FAR * szSqlStrIn,
+    SQLWCHAR * szSqlStrIn,
     SQLINTEGER cbSqlStrIn,
-    SQLWCHAR FAR * szSqlStr,
+    SQLWCHAR * szSqlStr,
     SQLINTEGER cbSqlStrMax,
-    SQLINTEGER FAR * pcbSqlStr)
+    SQLINTEGER * pcbSqlStr)
 {
   ENTER_HDBC (hdbc, 0,
     trace_SQLNativeSqlW (TRACE_ENTER,
