@@ -1,7 +1,7 @@
 /*
  *  hdbc.c
  *
- *  $Id: hdbc.c,v 1.22 2003/09/03 08:44:47 source Exp $
+ *  $Id: hdbc.c,v 1.27 2004/08/26 12:10:56 source Exp $
  *
  *  Data source connect object management functions
  *
@@ -91,10 +91,10 @@
 #include <stdio.h>
 
 
-extern SQLRETURN _iodbcdm_driverunload ();
+extern SQLRETURN _iodbcdm_driverunload (HDBC hdbc);
 
 static SQLRETURN
-_iodbcdm_drvopt_store (SQLHDBC hdbc, SQLUSMALLINT fOption, SQLUINTEGER vParam,
+_iodbcdm_drvopt_store (SQLHDBC hdbc, SQLUSMALLINT fOption, SQLULEN vParam,
 	SQLCHAR waMode)
 {
   CONN (pdbc, hdbc);
@@ -154,10 +154,10 @@ _iodbcdm_drvopt_free (SQLHDBC hdbc)
 SQLRETURN 
 SQLAllocConnect_Internal (
     SQLHENV henv,
-    SQLHDBC FAR * phdbc)
+    SQLHDBC * phdbc)
 {
   GENV (genv, henv);
-  DBC_t FAR *pdbc;
+  CONN (pdbc, NULL);
 
   if (phdbc == NULL)
     {
@@ -165,7 +165,7 @@ SQLAllocConnect_Internal (
       return SQL_ERROR;
     }
 
-  pdbc = (DBC_t FAR *) MEM_ALLOC (sizeof (DBC_t));
+  pdbc = (DBC_t *) MEM_ALLOC (sizeof (DBC_t));
 
   if (pdbc == NULL)
     {
@@ -182,7 +182,7 @@ SQLAllocConnect_Internal (
   pdbc->type = SQL_HANDLE_DBC;
 
   /* insert this dbc entry into the link list */
-  pdbc->next = genv->hdbc;
+  pdbc->next = (DBC_t *) genv->hdbc;
   genv->hdbc = pdbc;
 #if (ODBCVER >= 0x0300)
   if (genv->odbc_ver == 0)
@@ -219,7 +219,7 @@ SQLAllocConnect_Internal (
 
 
 SQLRETURN SQL_API
-SQLAllocConnect (SQLHENV henv, SQLHDBC FAR * phdbc)
+SQLAllocConnect (SQLHENV henv, SQLHDBC * phdbc)
 {
   GENV (genv, henv);
   SQLRETURN retcode = SQL_SUCCESS;
@@ -247,9 +247,9 @@ SQLAllocConnect (SQLHENV henv, SQLHDBC FAR * phdbc)
 SQLRETURN
 SQLFreeConnect_Internal (SQLHDBC hdbc)
 {
-  GENV_t FAR *genv;
   CONN (pdbc, hdbc);
-  DBC_t FAR *tpdbc;
+  GENV (genv, pdbc->genv);
+  CONN (tpdbc, NULL);
 
   /* check state */
   if (pdbc->state != en_dbc_allocated)
@@ -258,9 +258,7 @@ SQLFreeConnect_Internal (SQLHDBC hdbc)
       return SQL_ERROR;
     }
 
-  genv = (GENV_t FAR *) pdbc->genv;
-
-  for (tpdbc = (DBC_t FAR *) genv->hdbc; tpdbc != NULL; tpdbc = tpdbc->next)
+  for (tpdbc = (DBC_t *) genv->hdbc; tpdbc != NULL; tpdbc = tpdbc->next)
     {
       if (pdbc == tpdbc)
 	{
@@ -309,16 +307,16 @@ SQLFreeConnect (SQLHDBC hdbc)
 
 SQLRETURN
 _iodbcdm_SetConnectOption (
-    SQLHDBC hdbc,
-    SQLUSMALLINT fOption,
-    SQLUINTEGER vParam,
-    SQLCHAR waMode)
+  SQLHDBC		  hdbc,
+  SQLUSMALLINT		  fOption,
+  SQLULEN		  vParam,
+  SQLCHAR		  waMode)
 {
   CONN (pdbc, hdbc);
-  ENV_t FAR *penv = pdbc->henv;
-  STMT_t FAR *pstmt;
+  ENVR (penv, pdbc->henv);
+  STMT (pstmt, NULL);
   HPROC hproc = SQL_NULL_HPROC;
-  int sqlstat = en_00000;
+  sqlstcode_t sqlstat = en_00000;
   SQLRETURN retcode = SQL_SUCCESS;
 
 #if (ODBCVER < 0x0300)
@@ -373,9 +371,9 @@ _iodbcdm_SetConnectOption (
     }
 
   /* check state of statement handle(s) */
-  for (pstmt = (STMT_t FAR *) pdbc->hstmt;
+  for (pstmt = (STMT_t *) pdbc->hstmt;
       pstmt != NULL && sqlstat == en_00000;
-      pstmt = (STMT_t FAR *) pstmt->next)
+      pstmt = (STMT_t *) pstmt->next)
     {
       if (pstmt->state >= en_stmt_needdata || pstmt->asyn_on != en_NullProc)
 	{
@@ -436,9 +434,9 @@ _iodbcdm_SetConnectOption (
       SQLCHAR *_vParam;
       SQLCHAR *tmp = NULL;
 
-      if (((char FAR *)vParam) == NULL 
-          || (waMode != 'W' && ((char FAR *) vParam)[0] == '\0')
-          || (waMode == 'W' && ((wchar_t FAR *) vParam)[0] == L'\0'))
+      if (((char *)vParam) == NULL 
+          || (waMode != 'W' && ((char *) vParam)[0] == '\0')
+          || (waMode == 'W' && ((wchar_t *) vParam)[0] == L'\0'))
 	{
 	  PUSHSQLERR (pdbc->herr, en_S1009);
 	  return SQL_ERROR;
@@ -498,7 +496,7 @@ _iodbcdm_SetConnectOption (
               /* unicode=>ansi*/
                 _vParam = dm_SQL_W2A((SQLWCHAR *)vParam, SQL_NTS);
               }
-            vParam = (SQLUINTEGER)_vParam;
+            vParam = (SQLULEN)_vParam;
             break;
           }
       }
@@ -699,7 +697,7 @@ SQLRETURN SQL_API
 SQLSetConnectOption (
   SQLHDBC		  hdbc,
   SQLUSMALLINT		  fOption,
-  SQLUINTEGER		  vParam)
+  SQLULEN		  vParam)
 {
   ENTER_HDBC (hdbc, 1,
     trace_SQLSetConnectOption (TRACE_ENTER, hdbc, fOption, vParam));
@@ -715,7 +713,7 @@ SQLRETURN SQL_API
 SQLSetConnectOptionA (
   SQLHDBC		  hdbc,
   SQLUSMALLINT		  fOption,
-  SQLUINTEGER		  vParam)
+  SQLULEN		  vParam)
 {
   ENTER_HDBC (hdbc, 1,
     trace_SQLSetConnectOption (TRACE_ENTER, hdbc, fOption, vParam));
@@ -731,7 +729,7 @@ SQLRETURN SQL_API
 SQLSetConnectOptionW (
   SQLHDBC		  hdbc,
   SQLUSMALLINT		  fOption,
-  SQLUINTEGER		  vParam)
+  SQLULEN		  vParam)
 {
   ENTER_HDBC (hdbc, 1,
     trace_SQLSetConnectOptionW (TRACE_ENTER, hdbc, fOption, vParam));
@@ -751,9 +749,9 @@ _iodbcdm_GetConnectOption (
     SQLCHAR waMode)
 {
   CONN (pdbc, hdbc);
-  ENV_t FAR *penv = pdbc->henv;
+  ENVR (penv, pdbc->henv);
   HPROC hproc = SQL_NULL_HPROC;
-  int sqlstat = en_00000;
+  sqlstcode_t sqlstat = en_00000;
   SQLRETURN retcode = SQL_SUCCESS;
 
 #if (ODBCVER < 0x0300)
@@ -836,7 +834,7 @@ _iodbcdm_GetConnectOption (
         }
       else
         {
-           dm_strcpy_A2W (pvParam, (SQLCHAR *)fname);
+           dm_strcpy_A2W ((SQLWCHAR *)pvParam, (SQLCHAR *)fname);
         }
 
       free (fname);
@@ -986,12 +984,12 @@ _iodbcdm_GetConnectOption (
               if (waMode != 'W')
                 {
                  /* ansi<=unicode*/
-                  dm_StrCopyOut2_W2A (paramOut, pvParam, SQL_MAX_OPTION_STRING_LENGTH, NULL);
+                  dm_StrCopyOut2_W2A ((SQLWCHAR *)paramOut, (SQLCHAR *)pvParam, SQL_MAX_OPTION_STRING_LENGTH, NULL);
                 }
               else
                 {
                  /* unicode<=ansi*/
-                  dm_StrCopyOut2_A2W (paramOut, pvParam, SQL_MAX_OPTION_STRING_LENGTH, NULL);
+                  dm_StrCopyOut2_A2W ((SQLCHAR *)paramOut, (SQLWCHAR *)pvParam, SQL_MAX_OPTION_STRING_LENGTH, NULL);
                 }
               break;
             }
@@ -1081,7 +1079,7 @@ _iodbcdm_transact (
     UWORD fType)
 {
   CONN (pdbc, hdbc);
-  STMT_t FAR *pstmt;
+  STMT (pstmt, NULL);
   HPROC hproc;
   SQLRETURN retcode;
 
@@ -1101,7 +1099,7 @@ _iodbcdm_transact (
       break;
     }
 
-  for (pstmt = (STMT_t FAR *) (pdbc->hstmt);
+  for (pstmt = (STMT_t *) (pdbc->hstmt);
       pstmt != NULL;
       pstmt = pstmt->next)
     {
@@ -1148,7 +1146,7 @@ _iodbcdm_transact (
 
   pdbc->state = en_dbc_hstmt;
 
-  for (pstmt = (STMT_t FAR *) (pdbc->hstmt);
+  for (pstmt = (STMT_t *) (pdbc->hstmt);
       pstmt != NULL;
       pstmt = pstmt->next)
     {
@@ -1252,7 +1250,7 @@ SQLTransact_Internal (
     }
   else
     {
-      for (pdbc = (DBC_t FAR *) (genv->hdbc); pdbc != NULL; pdbc = pdbc->next)
+      for (pdbc = (DBC_t *) (genv->hdbc); pdbc != NULL; pdbc = pdbc->next)
 	{
 	  retcode |= _iodbcdm_transact (pdbc, fType);
 	}
