@@ -1,7 +1,7 @@
 /*
  *  henv.c
  *
- *  $Id: henv.c,v 1.13 2003/08/22 14:28:41 source Exp $
+ *  $Id: henv.c,v 1.15 2004/06/21 13:34:28 source Exp $
  *
  *  Environment object management functions
  *
@@ -95,15 +95,17 @@ SPINLOCK_DECLARE (iodbcdm_global_lock);
 #endif
 
 static int _iodbcdm_initialized = 0;
-static void Init_iODBC();
-static void Done_iODBC();
+static void Init_iODBC(void);
+static void Done_iODBC(void);
 
 
 static void
-_iodbcdm_env_settracing (GENV_t FAR *genv)
+_iodbcdm_env_settracing (GENV_t *genv)
 {
   char buf[1024];
 
+  genv = genv; /*UNUSED*/
+
   /*
    *  Check TraceFile keyword
    */
@@ -126,9 +128,9 @@ _iodbcdm_env_settracing (GENV_t FAR *genv)
 
 
 SQLRETURN 
-SQLAllocEnv_Internal (SQLHENV FAR * phenv, int odbc_ver)
+SQLAllocEnv_Internal (SQLHENV * phenv, int odbc_ver)
 {
-  GENV_t FAR *genv;
+  GENV (genv, NULL);
   int retcode = SQL_SUCCESS;
 
   /* 
@@ -171,14 +173,14 @@ SQLAllocEnv_Internal (SQLHENV FAR * phenv, int odbc_ver)
 
 
 SQLRETURN SQL_API
-SQLAllocEnv (SQLHENV FAR * phenv)
+SQLAllocEnv (SQLHENV * phenv)
 {
-  GENV_t *genv;
+  GENV (genv, NULL);
   int retcode = SQL_SUCCESS;
 
   retcode = SQLAllocEnv_Internal (phenv, SQL_OV_ODBC2);
 
-  genv = (GENV_t FAR *) *phenv;
+  genv = (GENV_t *) *phenv;
 
   /*
    * Start tracing
@@ -244,7 +246,7 @@ SQLFreeEnv (SQLHENV henv)
  *  properly
  */
 static void
-Init_iODBC ()
+Init_iODBC (void)
 {
 #if !defined (PTHREAD_MUTEX_INITIALIZER) || defined (WINDOWS)
   SPINLOCK_INIT (iodbcdm_global_lock);
@@ -269,7 +271,7 @@ Init_iODBC ()
 
 
 static void 
-Done_iODBC()
+Done_iODBC(void)
 {
     SPINLOCK_DONE (iodbcdm_global_lock);
 }
