@@ -1,7 +1,7 @@
 /*
  *  itrace.h
  *
- *  $Id: itrace.h,v 1.8 2003/08/22 14:28:41 source Exp $
+ *  $Id: itrace.h,v 1.10 2004/06/21 13:34:29 source Exp $
  *
  *  Trace functions
  *
@@ -97,22 +97,22 @@ extern int ODBCSharedTraceFlag;
 #define CALL_DRIVER_FUNC( hdbc, errHandle, ret, proc, plist ) \
     { \
 	ret = proc plist; \
-	if (errHandle) ((GENV_t FAR *)(errHandle))->rc = ret; \
+	if (errHandle) ((GENV_t *)(errHandle))->rc = ret; \
     }
 
 
 #define CALL_DRIVER( hdbc, errHandle, ret, proc, procid, plist ) \
     {\
-	DBC_t FAR*	pdbc = (DBC_t FAR*)(hdbc);\
-	ENV_t FAR*      penv = (ENV_t FAR*)(pdbc->henv);\
+	DBC_t *	t_pdbc = (DBC_t *)(hdbc);\
+	ENV_t * t_penv = (ENV_t *)(t_pdbc->henv);\
 \
-	if (!penv->thread_safe)\
-	    MUTEX_LOCK (penv->drv_lock);\
+	if (!t_penv->thread_safe)\
+	    MUTEX_LOCK (t_penv->drv_lock);\
 \
 	CALL_DRIVER_FUNC( hdbc, errHandle, ret, proc, plist )\
 \
-	if (!penv->thread_safe)\
-	    MUTEX_UNLOCK (penv->drv_lock);\
+	if (!t_penv->thread_safe)\
+	    MUTEX_UNLOCK (t_penv->drv_lock);\
     }
 
 #define CALL_UDRIVER(hdbc, errHandle, retcode, hproc, unicode_driver, procid, plist) \
