@@ -1,7 +1,7 @@
 /*
  *  odbc3.c
  *
- *  $Id: odbc3.c,v 1.13 2004/02/18 11:06:24 source Exp $
+ *  $Id: odbc3.c,v 1.19 2004/07/06 15:34:07 source Exp $
  *
  *  ODBC 3.x functions
  *
@@ -75,6 +75,7 @@
 
 #include <sql.h>
 #include <sqlext.h>
+#include <iodbcext.h>
 
 #if (ODBCVER >= 0x300)
 #include <sqlucode.h>
@@ -95,11 +96,11 @@
 /*
  *  Externals
  */
-SQLRETURN SQLAllocEnv_Internal (SQLHENV FAR * phenv, int odbc_ver);
+SQLRETURN SQLAllocEnv_Internal (SQLHENV * phenv, int odbc_ver);
 SQLRETURN SQLFreeEnv_Internal (SQLHENV henv);
-SQLRETURN SQLAllocConnect_Internal (SQLHENV henv, SQLHDBC FAR * phdbc);
+SQLRETURN SQLAllocConnect_Internal (SQLHENV henv, SQLHDBC * phdbc);
 SQLRETURN SQLFreeConnect_Internal (SQLHDBC hdbc);
-SQLRETURN SQLAllocStmt_Internal (SQLHDBC hdbc, SQLHSTMT FAR * phstmt);
+SQLRETURN SQLAllocStmt_Internal (SQLHDBC hdbc, SQLHSTMT * phstmt);
 SQLRETURN SQLFreeStmt_Internal (SQLHSTMT hstmt, SQLUSMALLINT fOption);
 SQLRETURN SQLTransact_Internal (SQLHENV henv, SQLHDBC hdbc, SQLUSMALLINT fType);
 
@@ -140,7 +141,7 @@ SQLAllocHandle_Internal (
 	CONN (con, inputHandle);
 	HPROC hproc = SQL_NULL_HPROC;
 	RETCODE retcode;
-	DESC_t FAR *new_desc;
+	DESC_t *new_desc;
 
 	if (((ENV_t *)(con->henv))->dodbc_ver == SQL_OV_ODBC2)
 	  {
@@ -161,7 +162,7 @@ SQLAllocHandle_Internal (
 	    return SQL_ERROR;
 	  }
 
-	new_desc = (DESC_t FAR *) MEM_ALLOC (sizeof (DESC_t));
+	new_desc = (DESC_t *) MEM_ALLOC (sizeof (DESC_t));
 	if (!new_desc)
 	  {
 	    PUSHSQLERR (con->herr, en_HY001);
@@ -184,7 +185,7 @@ SQLAllocHandle_Internal (
 	new_desc->desc_cip = 0;
 	*outputHandlePtr = new_desc;
 
-	new_desc->next = con->hdesc;
+	new_desc->next = (DESC_t *) con->hdesc;
 	con->hdesc = new_desc;
 
 	return SQL_SUCCESS;
@@ -351,7 +352,7 @@ _SQLFreeHandle_DESC (
   CONN (pdbc, pdesc->hdbc);
   HPROC hproc;
   RETCODE retcode = SQL_SUCCESS;
-  DESC_t FAR *curr_desc;
+  DESC_t *curr_desc;
 
   if (IS_VALID_HSTMT (pdesc->hstmt))
     {				/* the desc handle is implicit */
@@ -361,7 +362,7 @@ _SQLFreeHandle_DESC (
   CLEAR_ERRORS (pdesc);
 
   /* remove it from the dbc's list */
-  curr_desc = pdbc->hdesc;
+  curr_desc = (DESC_t *) pdbc->hdesc;
   while (curr_desc)
     {
       if (curr_desc == pdesc)
@@ -462,6 +463,8 @@ SQLSetEnvAttr_Internal (SQLHENV environmentHandle,
 {
   GENV (genv, environmentHandle);
 
+  StringLength = StringLength; /*UNUSED*/
+
   if (genv->hdbc)
     {
       PUSHSQLERR (genv->herr, en_HY010);
@@ -471,7 +474,7 @@ SQLSetEnvAttr_Internal (SQLHENV environmentHandle,
   switch (Attribute)
     {
     case SQL_ATTR_CONNECTION_POOLING:
-      switch ((SQLINTEGER) ValuePtr)
+      switch ((SQLINTEGER) (SQLULEN) ValuePtr)
 	{
 	case SQL_CP_OFF:
 	case SQL_CP_ONE_PER_DRIVER:
@@ -484,7 +487,7 @@ SQLSetEnvAttr_Internal (SQLHENV environmentHandle,
 	}
 
     case SQL_ATTR_CP_MATCH:
-      switch ((SQLINTEGER) ValuePtr)
+      switch ((SQLINTEGER) (SQLULEN) ValuePtr)
 	{
 	case SQL_CP_STRICT_MATCH:
 	case SQL_CP_RELAXED_MATCH:
@@ -496,11 +499,11 @@ SQLSetEnvAttr_Internal (SQLHENV environmentHandle,
 	}
 
     case SQL_ATTR_ODBC_VERSION:
-      switch ((SQLINTEGER) ValuePtr)
+      switch ((SQLINTEGER) (SQLULEN) ValuePtr)
 	{
 	case SQL_OV_ODBC2:
 	case SQL_OV_ODBC3:
-	  genv->odbc_ver = (SQLINTEGER) ValuePtr;
+	  genv->odbc_ver = (SQLINTEGER) (SQLULEN) ValuePtr;
 	  return (SQL_SUCCESS);
 
 	default:
@@ -509,7 +512,7 @@ SQLSetEnvAttr_Internal (SQLHENV environmentHandle,
 	}
 
     case SQL_ATTR_OUTPUT_NTS:
-      switch ((SQLINTEGER) ValuePtr)
+      switch ((SQLINTEGER) (SQLULEN) ValuePtr)
 	{
 	case SQL_TRUE:
 	  return SQL_SUCCESS;
@@ -570,7 +573,8 @@ SQLGetEnvAttr_Internal (
   if (Attribute != SQL_ATTR_CONNECTION_POOLING &&
       Attribute != SQL_ATTR_CP_MATCH &&
       Attribute != SQL_ATTR_ODBC_VERSION && 
-      Attribute != SQL_ATTR_OUTPUT_NTS)
+      Attribute != SQL_ATTR_OUTPUT_NTS &&
+      Attribute != SQL_ATTR_WCHAR_SIZE)
     {
       PUSHSQLERR (genv->herr, en_HY092);
       return SQL_ERROR;
@@ -601,6 +605,12 @@ SQLGetEnvAttr_Internal (
 	*((SQLINTEGER *) ValuePtr) = SQL_TRUE;
       return SQL_SUCCESS;
     }
+  if (Attribute == SQL_ATTR_WCHAR_SIZE)
+    {
+      if (ValuePtr)
+	*((SQLINTEGER *) ValuePtr) = sizeof(wchar_t);
+       return SQL_SUCCESS;
+     }
 
   /* fall back to the first driver */
   if (IS_VALID_HDBC (genv->hdbc))
@@ -683,10 +693,12 @@ SQLGetStmtAttr_Internal (
 {
   STMT (stmt, statementHandle);
   CONN (pdbc, stmt->hdbc);
-  ENV_t FAR *penv = pdbc->henv;
+  ENVR (penv, pdbc->henv);
   HPROC hproc = SQL_NULL_HPROC;
   SQLRETURN retcode = SQL_SUCCESS;
 
+  waMode = waMode; /*UNUSED*/
+
   switch (Attribute)
     {
     case SQL_ATTR_IMP_PARAM_DESC:
@@ -763,7 +775,7 @@ SQLGetStmtAttr_Internal (
 
     case SQL_ATTR_ROW_ARRAY_SIZE:
 
-      if (((ENV_t FAR *) ((DBC_t FAR *) stmt->hdbc)->henv)->dodbc_ver == SQL_OV_ODBC3)
+      if (((ENV_t *) ((DBC_t *) stmt->hdbc)->henv)->dodbc_ver == SQL_OV_ODBC3)
 	{
           CALL_UDRIVER(stmt->hdbc, stmt, retcode, hproc, 
             penv->unicode_driver, en_GetStmtAttr, (stmt->dhstmt, Attribute, 
@@ -804,7 +816,7 @@ SQLGetStmtAttr_Internal (
 
     case SQL_ATTR_FETCH_BOOKMARK_PTR:
 
-      if (((ENV_t FAR *) ((DBC_t FAR *) stmt->hdbc)->henv)->dodbc_ver == SQL_OV_ODBC3)
+      if (((ENV_t *) ((DBC_t *) stmt->hdbc)->henv)->dodbc_ver == SQL_OV_ODBC3)
 	{
           CALL_UDRIVER(stmt->hdbc, stmt, retcode, hproc, 
             penv->unicode_driver, en_GetStmtAttr, (stmt->dhstmt, Attribute, 
@@ -825,7 +837,7 @@ SQLGetStmtAttr_Internal (
 
     case SQL_ATTR_ROWS_FETCHED_PTR:
 
-      if (((ENV_t FAR *) ((DBC_t FAR *) stmt->hdbc)->henv)->dodbc_ver == SQL_OV_ODBC3)
+      if (((ENV_t *) ((DBC_t *) stmt->hdbc)->henv)->dodbc_ver == SQL_OV_ODBC3)
 	{
           CALL_UDRIVER(stmt->hdbc, stmt, retcode, hproc, 
             penv->unicode_driver, en_GetStmtAttr, (stmt->dhstmt, Attribute, 
@@ -846,7 +858,7 @@ SQLGetStmtAttr_Internal (
 
     case SQL_ATTR_METADATA_ID:
 
-      if (((ENV_t FAR *) ((DBC_t FAR *) stmt->hdbc)->henv)->dodbc_ver == SQL_OV_ODBC3)
+      if (((ENV_t *) ((DBC_t *) stmt->hdbc)->henv)->dodbc_ver == SQL_OV_ODBC3)
 	{
           CALL_UDRIVER(stmt->hdbc, stmt, retcode, hproc, 
             penv->unicode_driver, en_GetStmtAttr, (stmt->dhstmt, Attribute, 
@@ -867,7 +879,7 @@ SQLGetStmtAttr_Internal (
 
     case SQL_ATTR_PARAMS_PROCESSED_PTR:
 
-      if (((ENV_t FAR *) ((DBC_t FAR *) stmt->hdbc)->henv)->dodbc_ver == SQL_OV_ODBC3)
+      if (((ENV_t *) ((DBC_t *) stmt->hdbc)->henv)->dodbc_ver == SQL_OV_ODBC3)
 	{
           CALL_UDRIVER(stmt->hdbc, stmt, retcode, hproc, 
             penv->unicode_driver, en_GetStmtAttr, (stmt->dhstmt, Attribute, 
@@ -882,13 +894,13 @@ SQLGetStmtAttr_Internal (
       else
 	{			/* an ODBC2 driver */
 	  if (ValuePtr)
-	    *((SQLUINTEGER **) ValuePtr) = stmt->params_processed_ptr;
+	    *((SQLUINTEGER **) ValuePtr) = (SQLUINTEGER *) stmt->params_processed_ptr;
 	  return SQL_SUCCESS;
 	}
 
     case SQL_ATTR_PARAMSET_SIZE:
 
-      if (((ENV_t FAR *) ((DBC_t FAR *) stmt->hdbc)->henv)->dodbc_ver == SQL_OV_ODBC3)
+      if (((ENV_t *) ((DBC_t *) stmt->hdbc)->henv)->dodbc_ver == SQL_OV_ODBC3)
 	{
           CALL_UDRIVER(stmt->hdbc, stmt, retcode, hproc, 
             penv->unicode_driver, en_GetStmtAttr, (stmt->dhstmt, Attribute, 
@@ -909,7 +921,7 @@ SQLGetStmtAttr_Internal (
 
     case SQL_ATTR_ROW_STATUS_PTR:
 
-      if (((ENV_t FAR *) ((DBC_t FAR *) stmt->hdbc)->henv)->dodbc_ver == SQL_OV_ODBC3)
+      if (((ENV_t *) ((DBC_t *) stmt->hdbc)->henv)->dodbc_ver == SQL_OV_ODBC3)
 	{
           CALL_UDRIVER(stmt->hdbc, stmt, retcode, hproc, 
             penv->unicode_driver, en_GetStmtAttr, (stmt->dhstmt, Attribute, 
@@ -925,7 +937,7 @@ SQLGetStmtAttr_Internal (
 	{			/* an ODBC2 driver */
 	  if (ValuePtr)
 	    *((SQLUINTEGER **) ValuePtr) = stmt->row_status_allocated == SQL_FALSE ?
-		stmt->row_status_ptr :
+		(SQLUINTEGER *) stmt->row_status_ptr :
 		NULL;
 	  return SQL_SUCCESS;
 	}
@@ -945,7 +957,7 @@ SQLGetStmtAttr_Internal (
     case SQL_ATTR_USE_BOOKMARKS:
     case SQL_ATTR_MAX_LENGTH:
 
-      if (((ENV_t FAR *) ((DBC_t FAR *) stmt->hdbc)->henv)->dodbc_ver == SQL_OV_ODBC3)
+      if (((ENV_t *) ((DBC_t *) stmt->hdbc)->henv)->dodbc_ver == SQL_OV_ODBC3)
 	{
           CALL_UDRIVER(stmt->hdbc, stmt, retcode, hproc, 
             penv->unicode_driver, en_GetStmtAttr, (stmt->dhstmt, Attribute, 
@@ -1085,10 +1097,12 @@ SQLSetStmtAttr_Internal (
 {
   STMT (stmt, statementHandle);
   CONN (pdbc, stmt->hdbc);
-  ENV_t FAR *penv = pdbc->henv;
+  ENVR (penv, pdbc->henv);
   HPROC hproc = SQL_NULL_HPROC;
   SQLRETURN retcode = SQL_SUCCESS;
 
+  waMode = waMode; /*UNUSED*/
+
   if (stmt->state == en_stmt_needdata)
     {
       PUSHSQLERR (stmt->herr, en_HY010);
@@ -1144,7 +1158,7 @@ SQLSetStmtAttr_Internal (
 	  if (retcode != SQL_SUCCESS || retcode != SQL_SUCCESS_WITH_INFO)
 	    return SQL_ERROR;
 
-	  stmt->desc[APP_PARAM_DESC] = ValuePtr;
+	  stmt->desc[APP_PARAM_DESC] = (DESC_t *) ValuePtr;
 	  return SQL_SUCCESS;
 	}
 
@@ -1194,7 +1208,7 @@ SQLSetStmtAttr_Internal (
 	  if (retcode != SQL_SUCCESS || retcode != SQL_SUCCESS_WITH_INFO)
 	    return SQL_ERROR;
 
-	  stmt->desc[APP_ROW_DESC] = ValuePtr;
+	  stmt->desc[APP_ROW_DESC] = (DESC_t *) ValuePtr;
 	  return SQL_SUCCESS;
 	}
 
@@ -1222,7 +1236,7 @@ SQLSetStmtAttr_Internal (
 
     case SQL_ATTR_ROWS_FETCHED_PTR:
 
-      if (((ENV_t FAR *) ((DBC_t FAR *) stmt->hdbc)->henv)->dodbc_ver == SQL_OV_ODBC3)
+      if (((ENV_t *) ((DBC_t *) stmt->hdbc)->henv)->dodbc_ver == SQL_OV_ODBC3)
 	{
 
           CALL_UDRIVER(stmt->hdbc, stmt, retcode, hproc, 
@@ -1244,7 +1258,7 @@ SQLSetStmtAttr_Internal (
 
     case SQL_ATTR_FETCH_BOOKMARK_PTR:
 
-      if (((ENV_t FAR *) ((DBC_t FAR *) stmt->hdbc)->henv)->dodbc_ver == SQL_OV_ODBC3)
+      if (((ENV_t *) ((DBC_t *) stmt->hdbc)->henv)->dodbc_ver == SQL_OV_ODBC3)
 	{
           CALL_UDRIVER(stmt->hdbc, stmt, retcode, hproc, 
             penv->unicode_driver, en_SetStmtAttr, (stmt->dhstmt, Attribute, 
@@ -1265,7 +1279,7 @@ SQLSetStmtAttr_Internal (
 
     case SQL_ATTR_PARAMS_PROCESSED_PTR:
 
-      if (((ENV_t FAR *) ((DBC_t FAR *) stmt->hdbc)->henv)->dodbc_ver == SQL_OV_ODBC3)
+      if (((ENV_t *) ((DBC_t *) stmt->hdbc)->henv)->dodbc_ver == SQL_OV_ODBC3)
 	{
           CALL_UDRIVER(stmt->hdbc, stmt, retcode, hproc, 
             penv->unicode_driver, en_SetStmtAttr, (stmt->dhstmt, Attribute, 
@@ -1286,7 +1300,7 @@ SQLSetStmtAttr_Internal (
 
     case SQL_ATTR_PARAMSET_SIZE:
 
-      if (((ENV_t FAR *) ((DBC_t FAR *) stmt->hdbc)->henv)->dodbc_ver == SQL_OV_ODBC3)
+      if (((ENV_t *) ((DBC_t *) stmt->hdbc)->henv)->dodbc_ver == SQL_OV_ODBC3)
 	{
           CALL_UDRIVER(stmt->hdbc, stmt, retcode, hproc, 
             penv->unicode_driver, en_SetStmtAttr, (stmt->dhstmt, Attribute, 
@@ -1301,13 +1315,13 @@ SQLSetStmtAttr_Internal (
 	}
       else
 	{			/* an ODBC2 driver */
-	  stmt->paramset_size = (SQLUINTEGER) ValuePtr;
+	  stmt->paramset_size = (SQLUINTEGER) (SQLULEN) ValuePtr;
 	  return SQL_SUCCESS;
 	}
 
     case SQL_ATTR_ROW_ARRAY_SIZE:
 
-      if (((ENV_t FAR *) ((DBC_t FAR *) stmt->hdbc)->henv)->dodbc_ver == SQL_OV_ODBC3)
+      if (((ENV_t *) ((DBC_t *) stmt->hdbc)->henv)->dodbc_ver == SQL_OV_ODBC3)
 	{
           CALL_UDRIVER(stmt->hdbc, stmt, retcode, hproc, 
             penv->unicode_driver, en_SetStmtAttr, (stmt->dhstmt, Attribute, 
@@ -1333,12 +1347,12 @@ SQLSetStmtAttr_Internal (
 	}
       else
 	{			/* an ODBC2 driver */
-	  if ((SQLUINTEGER) ValuePtr < 1)
+	  if ((SQLUINTEGER) (SQLULEN) ValuePtr < 1)
 	    {
 	      PUSHSQLERR (stmt->herr, en_HY024);
 	      return SQL_ERROR;
 	    }
-	  stmt->row_array_size = (SQLUINTEGER) ValuePtr;
+	  stmt->row_array_size = (SQLUINTEGER) (SQLULEN) ValuePtr;
 
 	  /* reallocate the row_status_ptr */
 	  if (stmt->row_status_ptr && stmt->row_status_allocated == SQL_TRUE)
@@ -1407,7 +1421,7 @@ SQLSetStmtAttr_Internal (
 
     case SQL_ATTR_ROW_STATUS_PTR:
 
-      if (((ENV_t FAR *) ((DBC_t FAR *) stmt->hdbc)->henv)->dodbc_ver == SQL_OV_ODBC3)
+      if (((ENV_t *) ((DBC_t *) stmt->hdbc)->henv)->dodbc_ver == SQL_OV_ODBC3)
 	{
           CALL_UDRIVER(stmt->hdbc, stmt, retcode, hproc, 
             penv->unicode_driver, en_SetStmtAttr, (stmt->dhstmt, Attribute, 
@@ -1447,7 +1461,7 @@ SQLSetStmtAttr_Internal (
     case SQL_ROWSET_SIZE:
     case SQL_ATTR_MAX_LENGTH:
 
-      if (((ENV_t FAR *) ((DBC_t FAR *) stmt->hdbc)->henv)->dodbc_ver == SQL_OV_ODBC3)
+      if (((ENV_t *) ((DBC_t *) stmt->hdbc)->henv)->dodbc_ver == SQL_OV_ODBC3)
 	{
           CALL_UDRIVER(stmt->hdbc, stmt, retcode, hproc, 
             penv->unicode_driver, en_SetStmtAttr, (stmt->dhstmt, Attribute, 
@@ -1621,7 +1635,7 @@ SQLSetConnectAttr_Internal (
   SQLCHAR		  waMode)
 {
   CONN (con, connectionHandle);
-  ENV_t FAR *penv = con->henv;
+  ENVR (penv, con->henv);
   HPROC hproc = SQL_NULL_HPROC;
   SQLRETURN retcode = SQL_SUCCESS;
   void * _ValuePtr = NULL;
@@ -1646,13 +1660,13 @@ SQLSetConnectAttr_Internal (
           if (waMode != 'W')
             {
             /* ansi=>unicode*/
-              _ValuePtr = dm_SQL_A2W(ValuePtr, StringLength);
+              _ValuePtr = dm_SQL_A2W((SQLCHAR *) ValuePtr, StringLength);
             }
           else
             {
             /* unicode=>ansi*/
-              StringLength = (StringLength != SQL_NTS ? StringLength / sizeof(wchar_t) : SQL_NTS);
-              _ValuePtr = dm_SQL_W2A(ValuePtr, StringLength);
+              StringLength = (StringLength != SQL_NTS) ? (SQLINTEGER) (StringLength / sizeof(wchar_t)) : SQL_NTS;
+              _ValuePtr = dm_SQL_W2A((SQLWCHAR *) ValuePtr, StringLength);
             }
           ValuePtr = _ValuePtr;
           StringLength = SQL_NTS;
@@ -1676,7 +1690,7 @@ SQLSetConnectAttr_Internal (
 
     default:
       retcode = _iodbcdm_SetConnectOption (con, Attribute, 
-      	  (SQLUINTEGER) ValuePtr, waMode);
+      	  (SQLULEN) ValuePtr, waMode);
       return retcode;
     }
 }
@@ -1773,7 +1787,7 @@ SQLGetConnectAttr_Internal (
   SQLCHAR 		  waMode)
 {
   CONN (con, connectionHandle);
-  ENV_t FAR *penv = con->henv;
+  ENVR (penv, con->henv);
   HPROC hproc = SQL_NULL_HPROC;
   RETCODE retcode = SQL_SUCCESS;
   void * _Value = NULL;
@@ -1841,8 +1855,9 @@ SQLGetConnectAttr_Internal (
                 /* ansi<=unicode*/
 		  SQLSMALLINT retlen;
 
-                  dm_StrCopyOut2_W2A (valueOut, ValuePtr, 
-                    StringLength / sizeof(wchar_t), &retlen);
+                  dm_StrCopyOut2_W2A ((SQLWCHAR *) valueOut,
+			(SQLCHAR *) ValuePtr, 
+			StringLength / sizeof(wchar_t), &retlen);
                   if (StringLengthPtr)
                     *StringLengthPtr = retlen;
                 }
@@ -1851,8 +1866,9 @@ SQLGetConnectAttr_Internal (
                 /* unicode<=ansi*/
 		  SQLSMALLINT retlen;
 
-                  dm_StrCopyOut2_A2W (valueOut, ValuePtr, StringLength, 
-                    &retlen);
+                  dm_StrCopyOut2_A2W ((SQLCHAR *) valueOut, 
+			(SQLWCHAR *) ValuePtr, 
+			StringLength, &retlen);
                   if (StringLengthPtr)
                     *StringLengthPtr = retlen * sizeof(wchar_t);
                 }
@@ -1981,7 +1997,7 @@ SQLGetDescField_Internal (
 {
   DESC (desc, descriptorHandle);
   CONN (pdbc, desc->hdbc);
-  ENV_t FAR *penv = pdbc->henv;
+  ENVR (penv, pdbc->henv);
   HPROC hproc = SQL_NULL_HPROC;
   SQLRETURN retcode = SQL_SUCCESS;
   void * valueOut = ValuePtr;
@@ -2062,8 +2078,9 @@ SQLGetDescField_Internal (
             /* ansi<=unicode*/
 	      SQLSMALLINT retlen;
 
-              dm_StrCopyOut2_W2A (valueOut, ValuePtr, 
-                 BufferLength / sizeof(wchar_t), &retlen);
+              dm_StrCopyOut2_W2A ((SQLWCHAR *) valueOut, 
+		(SQLCHAR *) ValuePtr, 
+		BufferLength / sizeof(wchar_t), &retlen);
 	      if (StringLengthPtr)
                 *StringLengthPtr = retlen;
             }
@@ -2072,8 +2089,9 @@ SQLGetDescField_Internal (
             /* unicode<=ansi*/
 	      SQLSMALLINT retlen;
 
-              dm_StrCopyOut2_A2W (valueOut, ValuePtr, BufferLength, 
-                &retlen);
+              dm_StrCopyOut2_A2W ((SQLCHAR *) valueOut, 
+		(SQLWCHAR *) ValuePtr, 
+		BufferLength, &retlen);
               if (StringLengthPtr)
                 *StringLengthPtr = retlen * sizeof(wchar_t);
             }
@@ -2194,7 +2212,7 @@ SQLSetDescField_Internal (
 {
   DESC (desc, descriptorHandle);
   CONN (pdbc, desc->hdbc);
-  ENV_t FAR *penv = pdbc->henv;
+  ENVR (penv, pdbc->henv);
   HPROC hproc = SQL_NULL_HPROC;
   SQLRETURN retcode = SQL_SUCCESS;
   void * _ValuePtr = NULL;
@@ -2218,13 +2236,13 @@ SQLSetDescField_Internal (
           if (waMode != 'W')
             {
             /* ansi=>unicode*/
-              _ValuePtr = dm_SQL_A2W(ValuePtr, BufferLength);
+              _ValuePtr = dm_SQL_A2W((SQLCHAR *) ValuePtr, BufferLength);
             }
           else
             {
             /* unicode=>ansi*/
-              BufferLength = (BufferLength != SQL_NTS ? BufferLength / sizeof(wchar_t) : SQL_NTS);
-              _ValuePtr = dm_SQL_W2A(ValuePtr, BufferLength);
+              BufferLength = (BufferLength != SQL_NTS) ? (SQLINTEGER) (BufferLength / sizeof(wchar_t)) : SQL_NTS;
+              _ValuePtr = dm_SQL_W2A((SQLWCHAR *) ValuePtr, BufferLength);
             }
           ValuePtr = _ValuePtr;
           BufferLength = SQL_NTS;
@@ -2325,7 +2343,7 @@ SQLGetDescRec_Internal (
   SQLSMALLINT		* StringLengthPtr,
   SQLSMALLINT		* TypePtr,
   SQLSMALLINT 		* SubTypePtr,
-  SQLINTEGER		* LengthPtr,
+  SQLLEN		* LengthPtr,
   SQLSMALLINT		* PrecisionPtr,
   SQLSMALLINT		* ScalePtr,
   SQLSMALLINT		* NullablePtr,
@@ -2333,7 +2351,7 @@ SQLGetDescRec_Internal (
 {
   DESC (desc, descriptorHandle);
   CONN (pdbc, desc->hdbc);
-  ENV_t FAR *penv = pdbc->henv;
+  ENVR (penv, pdbc->henv);
   HPROC hproc = SQL_NULL_HPROC;
   SQLRETURN retcode = SQL_SUCCESS;
   void * nameOut = Name;
@@ -2383,12 +2401,12 @@ SQLGetDescRec_Internal (
       if (waMode != 'W')
         {
         /* ansi<=unicode*/
-          dm_StrCopyOut2_W2A (nameOut, Name, BufferLength, StringLengthPtr);
+          dm_StrCopyOut2_W2A ((SQLWCHAR *) nameOut, (SQLCHAR *) Name, BufferLength, StringLengthPtr);
         }
       else
         {
         /* unicode<=ansi*/
-          dm_StrCopyOut2_A2W (nameOut, Name, BufferLength, StringLengthPtr);
+          dm_StrCopyOut2_A2W ((SQLCHAR *) nameOut, (SQLWCHAR *) Name, BufferLength, StringLengthPtr);
         }
     }
 
@@ -2407,7 +2425,7 @@ SQLGetDescRec (
   SQLSMALLINT		* StringLengthPtr,
   SQLSMALLINT		* TypePtr,
   SQLSMALLINT		* SubTypePtr,
-  SQLINTEGER		* LengthPtr,
+  SQLLEN		* LengthPtr,
   SQLSMALLINT		* PrecisionPtr,
   SQLSMALLINT		* ScalePtr,
   SQLSMALLINT		* NullablePtr)
@@ -2459,7 +2477,7 @@ SQLGetDescRecA (
   SQLSMALLINT		* StringLengthPtr,
   SQLSMALLINT		* TypePtr,
   SQLSMALLINT		* SubTypePtr,
-  SQLINTEGER		* LengthPtr,
+  SQLLEN		* LengthPtr,
   SQLSMALLINT		* PrecisionPtr,
   SQLSMALLINT		* ScalePtr,
   SQLSMALLINT		* NullablePtr)
@@ -2511,7 +2529,7 @@ SQLGetDescRecW (
   SQLSMALLINT		* StringLengthPtr,
   SQLSMALLINT		* TypePtr,
   SQLSMALLINT		* SubTypePtr,
-  SQLINTEGER		* LengthPtr,
+  SQLLEN		* LengthPtr,
   SQLSMALLINT		* PrecisionPtr,
   SQLSMALLINT		* ScalePtr,
   SQLSMALLINT		* NullablePtr)
@@ -2560,12 +2578,12 @@ SQLSetDescRec_Internal (
   SQLSMALLINT		  RecNumber,
   SQLSMALLINT		  Type,
   SQLSMALLINT		  SubType,
-  SQLINTEGER		  Length,
+  SQLLEN		  Length,
   SQLSMALLINT		  Precision,
   SQLSMALLINT		  Scale,
   SQLPOINTER		  Data,
-  SQLINTEGER		* StringLength,
-  SQLINTEGER		* Indicator)
+  SQLLEN		* StringLength,
+  SQLLEN		* Indicator)
 {
   DESC (desc, DescriptorHandle);
   HPROC hproc;
@@ -2585,19 +2603,18 @@ SQLSetDescRec_Internal (
   return retcode;
 }
 
-
 RETCODE SQL_API
 SQLSetDescRec (
   SQLHDESC		  DescriptorHandle,
   SQLSMALLINT		  RecNumber,
   SQLSMALLINT		  Type,
   SQLSMALLINT		  SubType,
-  SQLINTEGER		  Length,
+  SQLLEN		  Length,
   SQLSMALLINT		  Precision,
   SQLSMALLINT		  Scale,
   SQLPOINTER		  Data,
-  SQLINTEGER		* StringLength,
-  SQLINTEGER		* Indicator)
+  SQLLEN		* StringLength,
+  SQLLEN		* Indicator)
 {
   ENTER_DESC (DescriptorHandle,
     trace_SQLSetDescRec (TRACE_ENTER,
@@ -2668,13 +2685,13 @@ SQLColAttribute_Internal (
   SQLPOINTER		  CharacterAttributePtr,
   SQLSMALLINT		  BufferLength,
   SQLSMALLINT		* StringLengthPtr,
-  SQLPOINTER		  NumericAttributePtr,
+  SQLLEN		* NumericAttributePtr,
   SQLCHAR		  waMode)
 {
   STMT (stmt, statementHandle);
   CONN (pdbc, stmt->hdbc);
-  ENV_t FAR *penv = pdbc->henv;
-  GENV_t FAR *genv = pdbc->genv;
+  ENVR (penv, pdbc->henv);
+  GENV (genv, pdbc->genv);
   HPROC hproc = SQL_NULL_HPROC;
   SQLRETURN retcode = SQL_SUCCESS;
   void * charAttrOut = CharacterAttributePtr;
@@ -2762,14 +2779,16 @@ SQLColAttribute_Internal (
               if (waMode != 'W')
                 {
                 /* ansi<=unicode*/
-                  dm_StrCopyOut2_W2A (charAttrOut, CharacterAttributePtr, 
-                    BufferLength / sizeof(wchar_t), StringLengthPtr);
+                  dm_StrCopyOut2_W2A ((SQLWCHAR *) charAttrOut, 
+			(SQLCHAR *) CharacterAttributePtr,
+			BufferLength / sizeof(wchar_t), StringLengthPtr);
                 }
               else
                 {
                 /* unicode<=ansi*/
-                  dm_StrCopyOut2_A2W (charAttrOut, CharacterAttributePtr, 
-                    BufferLength, StringLengthPtr);
+                  dm_StrCopyOut2_A2W ((SQLCHAR *) charAttrOut, 
+			(SQLWCHAR *) CharacterAttributePtr,
+			BufferLength, StringLengthPtr);
                   if (StringLengthPtr)
                     *StringLengthPtr = *StringLengthPtr * sizeof(wchar_t);
                 }
@@ -2867,7 +2886,7 @@ SQLColAttribute_Internal (
 	  if (CharacterAttributePtr)
 	    {
 	      STRNCPY (CharacterAttributePtr, szval, len1);
-	      ((SQLCHAR FAR *) CharacterAttributePtr)[len1] = 0;
+	      ((SQLCHAR *) CharacterAttributePtr)[len1] = 0;
 	    }
 	  if (StringLengthPtr)
 	    *StringLengthPtr = len;
@@ -2969,14 +2988,16 @@ SQLColAttribute_Internal (
               if (waMode != 'W')
                 {
                 /* ansi<=unicode*/
-                  dm_StrCopyOut2_W2A (charAttrOut, CharacterAttributePtr, 
-                    BufferLength / sizeof(wchar_t), StringLengthPtr);
+                  dm_StrCopyOut2_W2A ((SQLWCHAR *) charAttrOut, 
+			(SQLCHAR *) CharacterAttributePtr,
+			BufferLength / sizeof(wchar_t), StringLengthPtr);
                 }
               else
                 {
                 /* unicode<=ansi*/
-                  dm_StrCopyOut2_A2W (charAttrOut, CharacterAttributePtr, 
-                    BufferLength, StringLengthPtr);
+                  dm_StrCopyOut2_A2W ((SQLCHAR *) charAttrOut,
+			(SQLWCHAR *) CharacterAttributePtr,
+			BufferLength, StringLengthPtr);
                   if (StringLengthPtr)
                     *StringLengthPtr = *StringLengthPtr * sizeof(wchar_t);
                 }
@@ -2996,7 +3017,7 @@ SQLColAttribute (
   SQLPOINTER		  CharacterAttributePtr,
   SQLSMALLINT		  BufferLength,
   SQLSMALLINT		* StringLengthPtr,
-  SQLPOINTER 		  NumericAttributePtr)
+  SQLLEN		* NumericAttributePtr)
 {
   ENTER_STMT (statementHandle,
     trace_SQLColAttribute (TRACE_ENTER, 
@@ -3031,7 +3052,7 @@ SQLColAttributeA (
   SQLPOINTER		  CharacterAttributePtr,
   SQLSMALLINT		  BufferLength,
   SQLSMALLINT		* StringLengthPtr,
-  SQLPOINTER 		  NumericAttributePtr)
+  SQLLEN		* NumericAttributePtr)
 {
   ENTER_STMT (statementHandle,
     trace_SQLColAttribute (TRACE_ENTER, 
@@ -3066,7 +3087,7 @@ SQLColAttributeW (
   SQLPOINTER		  CharacterAttributePtr,
   SQLSMALLINT		  BufferLength,
   SQLSMALLINT		* StringLengthPtr,
-  SQLPOINTER		  NumericAttributePtr)
+  SQLLEN		* NumericAttributePtr)
 {
   ENTER_STMT (statementHandle,
     trace_SQLColAttributeW (TRACE_ENTER, 
@@ -3202,7 +3223,7 @@ static RETCODE
 SQLFetchScroll_Internal (
   SQLHSTMT		  statementHandle,
   SQLSMALLINT		  fetchOrientation,
-  SQLINTEGER		  fetchOffset)
+  SQLLEN		  fetchOffset)
 {
   STMT (stmt, statementHandle);
   HPROC hproc;
@@ -3271,15 +3292,15 @@ SQLFetchScroll_Internal (
 	      return SQL_ERROR;
 	    }
 	  retcode = _iodbcdm_ExtendedFetch (statementHandle, fetchOrientation,
-	      stmt->
-	      fetch_bookmark_ptr ? *((SQLINTEGER *) stmt->fetch_bookmark_ptr)
-	      : 0, stmt->rows_fetched_ptr, stmt->row_status_ptr);
+	      stmt->fetch_bookmark_ptr ? *((SQLINTEGER *) stmt->fetch_bookmark_ptr)
+	      : 0, (SQLULEN *) stmt->rows_fetched_ptr, 
+		(SQLUSMALLINT *) stmt->row_status_ptr);
 	}
       else
 	retcode =
 	    _iodbcdm_ExtendedFetch (statementHandle, fetchOrientation,
-	    fetchOffset, stmt->rows_fetched_ptr, stmt->row_status_ptr);
-
+	    fetchOffset, (SQLULEN *) stmt->rows_fetched_ptr, 
+	    (SQLUSMALLINT *) stmt->row_status_ptr);
     }
 
   /* state transition */
@@ -3334,7 +3355,7 @@ RETCODE SQL_API
 SQLFetchScroll (
   SQLHSTMT		  StatementHandle,
   SQLSMALLINT		  FetchOrientation,
-  SQLINTEGER		  FetchOffset)
+  SQLLEN		  FetchOffset)
 {
   ENTER_STMT (StatementHandle,
     trace_SQLFetchScroll (TRACE_ENTER,
@@ -3348,7 +3369,7 @@ SQLFetchScroll (
 	FetchOffset);
 
   if (retcode == SQL_SUCCESS || retcode == SQL_SUCCESS_WITH_INFO)
-    _iodbcdm_ConvBindData (StatementHandle);
+    _iodbcdm_ConvBindData ((STMT_t *) StatementHandle);
 
   LEAVE_STMT (StatementHandle,
     trace_SQLFetchScroll (TRACE_LEAVE,
@@ -3364,10 +3385,10 @@ SQLBindParam (
     SQLUSMALLINT ipar,
     SQLSMALLINT fCType,
     SQLSMALLINT fSqlType,
-    SQLUINTEGER cbParamDef,
+    SQLULEN cbParamDef,
     SQLSMALLINT ibScale,
     SQLPOINTER rgbValue,
-    SQLINTEGER FAR * pcbValue)
+    SQLLEN *pcbValue)
 {
   return SQLBindParameter (hstmt, ipar, SQL_PARAM_INPUT, fCType, fSqlType, cbParamDef, ibScale, rgbValue, SQL_MAX_OPTION_STRING_LENGTH, pcbValue);
 }
@@ -3377,11 +3398,11 @@ static SQLRETURN
 SQLCloseCursor_Internal (SQLHSTMT hstmt)
 {
   STMT (pstmt, hstmt);
-  DBC_t FAR *pdbc;
+  CONN (pdbc, NULL);
   HPROC hproc = SQL_NULL_HPROC;
   SQLRETURN retcode = SQL_SUCCESS;
 
-  pdbc = (DBC_t FAR *) (pstmt->hdbc);
+  pdbc = (DBC_t *) (pstmt->hdbc);
 
   /* check state */
   if (pstmt->state >= en_stmt_needdata || pstmt->asyn_on != en_NullProc)
