@@ -1,7 +1,7 @@
 /*
  *  DriverConnect.c
  *
- *  $Id: DriverConnect.c,v 1.2 2003/08/22 14:28:41 source Exp $
+ *  $Id: DriverConnect.c,v 1.3 2004/06/21 13:32:47 source Exp $
  *
  *  SQLDriverConnect trace functions
  *
@@ -143,7 +143,7 @@ _trace_connstr (
   int 			  output)
 {
   SQLCHAR *dup;
-  long length;
+  ssize_t length;
 
   if (!str)
     {
@@ -171,7 +171,7 @@ _trace_connstr (
   /*
    *  Make a copy of the string
    */
-  if ((dup = malloc (length + 1)) == NULL)
+  if ((dup = (SQLCHAR *) malloc (length + 1)) == NULL)
     return;
   memcpy (dup, str, length);
   dup[length] = '\0';
