@@ -1,7 +1,7 @@
 /*
  *  Info.c
  *
- *  $Id: Info.c,v 1.2 2003/08/22 14:28:41 source Exp $
+ *  $Id: Info.c,v 1.3 2004/06/21 13:32:47 source Exp $
  *
  *  Decode the SQLGetInfo responses and dump them to the trace log
  *
@@ -1266,6 +1266,9 @@ _trace_getinfo (
   int elem;
   int i;
 
+  cbInfoValueMax = cbInfoValueMax;	/*UNUSED*/
+  pcbInfoValue = pcbInfoValue;		/*UNUSED*/
+
   /*
    *  If the pointer is NULL, we have no information to decode, so
    *  we just print the generic details.
