@@ -1,19 +1,19 @@
 /*
  *  trace.c
  *
- *  $Id: trace.c,v 1.3 2004/02/17 19:56:34 source Exp $
+ *  $Id: trace.c,v 1.7 2004/07/06 15:26:07 source Exp $
  *
  *  Trace functions
  *
  *  The iODBC driver manager.
- *  
+ *
  *  Copyright (C) 1996-2003 by OpenLink Software <iodbc@openlinksw.com>
  *  All Rights Reserved.
  *
  *  This software is released under the terms of either of the following
  *  licenses:
  *
- *      - GNU Library General Public License (see LICENSE.LGPL) 
+ *      - GNU Library General Public License (see LICENSE.LGPL)
  *      - The BSD License (see LICENSE.BSD).
  *
  *  While not mandated by the BSD license, any patches you make to the
@@ -87,12 +87,12 @@
 #include "trace.h"
 #include "unicode.h"
 
-
+#define NO_CARBON 1
 #if defined(macintosh)
 # include       <Errors.h>
 # include       <OSUtils.h>
 # include       <Processes.h>
-#elif defined(__APPLE__)
+#elif defined(__APPLE__) && !defined(NO_CARBON)
 # include       <Carbon/Carbon.h>
 #endif
 
@@ -121,7 +121,7 @@ trace_set_appname (char *appname)
 }
 
 
-char * 
+char *
 trace_get_filename (void)
 {
   return STRDUP (trace_fname);
@@ -134,7 +134,7 @@ trace_set_filename (char *fname)
   char buf[255];
   char *s, *p;
   struct passwd *pwd;
-  int i;
+  size_t i;
   MEM_FREE (trace_fname);
 
   memset (buf, '\0', sizeof (buf));
@@ -196,7 +196,7 @@ trace_set_filename (char *fname)
 		  s += 2;
 		  continue;
 		  }
-	        
+
 
 	    default:
 	       buf[i++] = *s++;
@@ -210,7 +210,7 @@ trace_set_filename (char *fname)
 }
 
 
-void 
+void
 trace_start(void)
 {
   /*
@@ -277,7 +277,7 @@ trace_start(void)
     extern char *__progname;
     trace_set_appname (__progname);
   }
-#elif defined(macintosh) || defined(__APPLE__)
+#elif defined(macintosh) || (defined(__APPLE__) && !defined(NO_CARBON))
   {
     ProcessSerialNumber PSN;
     ProcessInfoRec prec;
@@ -359,7 +359,7 @@ trace_emit (char *fmt, ...)
 void
 trace_emit_string (SQLCHAR *str, int len, int is_utf8)
 {
-  long length = len;
+  ssize_t length = len;
   int i, j;
   long col;
   SQLCHAR *ptr;
@@ -417,14 +417,30 @@ trace_emit_string (SQLCHAR *str, int len, int is_utf8)
 	bytes = 3;
       else if ((c & 0xF8) == 0xF0)
 	bytes = 4;
+      else if ((c & 0xFC) == 0xF8)
+        bytes = 5;
+      else if ((c & 0xFE) == 0xFC)
+        bytes = 6;
       else
-	bytes = -1;
+	bytes = -1;	/* Wrong UTF8 character */
 
-      /*
-       *  Emit the number of bytes calculated
-       */
-      for (j = 0; j < bytes; j++)
-	trace_emitc (*ptr++);
+      if (bytes > 0)
+        {
+	  /*
+	   *  Emit the number of bytes calculated
+	   */
+	  for (j = 0; j < bytes; j++)
+	    trace_emitc (*ptr++);
+	}
+      else
+        {
+	  /*
+	   *  Skip this bogus UTF8 character sequence and emit a single # 
+	   */
+	  for (bytes = 1, ptr++; (*ptr & 0xC0) == 0x80; bytes++)
+	    ptr++;	
+	  trace_emitc ('#');
+	}
 
       /*
        *  After 40 characters, start a new line
@@ -482,7 +498,7 @@ trace_emit_binary (unsigned char *str, int len)
     {
       unsigned char c = *ptr++;
 
-      /* 
+      /*
        *  Put data into buffer
        */
       buf[3 * col] = HEX[(c >> 4) & 0xF];
@@ -537,11 +553,11 @@ _trace_print_function (int func, int trace_leave, int retcode)
 
   if (trace_leave == TRACE_LEAVE)
     trace_emit ("\n%-15.15s %08lX EXIT  %s with return code %d (%s)\n",
-	trace_appname ? trace_appname : "Application", 
+	trace_appname ? trace_appname : "Application",
 	THREAD_IDENT, odbcapi_symtab[func], retcode, ptr);
   else
     trace_emit ("\n%-15.15s %08lX ENTER %s\n",
-	trace_appname ? trace_appname : "Application", 
+	trace_appname ? trace_appname : "Application",
 	THREAD_IDENT, odbcapi_symtab[func]);
 }
 
@@ -696,6 +712,59 @@ _trace_stringlen (char *type, SQLINTEGER len)
     trace_emit ("\t\t%-15.15s   %ld\n", type, (long) len);
 }
 
+void
+_trace_len (SQLLEN i)
+{
+#ifdef _WIN64
+  trace_emit ("\t\t%-15.15s   %I64d\n", "SQLLEN", (INT64) i);
+#else
+  trace_emit ("\t\t%-15.15s   %ld\n", "SQLLEN", (long) i);
+#endif
+}
+
+
+void
+_trace_ulen (SQLULEN i)
+{
+#ifdef _WIN64
+  trace_emit ("\t\t%-15.15s   %I64u\n", "SQLLEN", (UINT64) i);
+#else
+  trace_emit ("\t\t%-15.15s   %ld\n", "SQLLEN", (unsigned long) i);
+#endif
+}
+
+
+void
+_trace_len_p (SQLLEN *p, int output)
+{
+  if (!p)
+    trace_emit ("\t\t%-15.15s * 0x0\n", "SQLLEN");
+  else if (output)
+#ifdef _WIN64
+    trace_emit ("\t\t%-15.15s * %p (%I64d)\n", "SQLLEN", p, (INT64) *p);
+#else
+    trace_emit ("\t\t%-15.15s * %p (%ld)\n", "SQLLEN", p, (long) *p);
+#endif
+  else
+    trace_emit ("\t\t%-15.15s * %p\n", "SQLLEN", p);
+}
+
+
+void
+_trace_ulen_p (SQLULEN *p, int output)
+{
+  if (!p)
+    trace_emit ("\t\t%-15.15s * 0x0\n", "SQLULEN");
+  else if (output)
+#ifdef _WIN64
+    trace_emit ("\t\t%-15.15s * %p (%I64u)\n", "SQLULEN", p, (UINT64) *p);
+#else
+    trace_emit ("\t\t%-15.15s * %p (%lu)\n", "SQLULEN", p, (unsigned long) *p);
+#endif
+  else
+    trace_emit ("\t\t%-15.15s * %p\n", "SQLULEN", p);
+}
+
 
 void
 _trace_string (SQLCHAR * str, SQLSMALLINT len, SQLSMALLINT * lenptr, int output)
@@ -1038,7 +1107,7 @@ _trace_bufferlen (SQLINTEGER length)
    */
   if (length <= SQL_LEN_BINARY_ATTR_OFFSET)
     {
-        sprintf (buf, "SQL_LEN_BINARY_ATTR(%ld)", SQL_LEN_BINARY_ATTR(length));
+        sprintf (buf, "SQL_LEN_BINARY_ATTR(%ld)", (long) SQL_LEN_BINARY_ATTR(length));
 	ptr = buf;
     }
 
